@@ -6,6 +6,7 @@ from emora_stdm.state_transition_dialogue_manager.memory import Memory
 from emora_stdm.state_transition_dialogue_manager.utilities import HashableSet, HashableDict, ConfigurationDict
 from emora_stdm.state_transition_dialogue_manager.natex_nlg import NatexNLG
 from emora_stdm.state_transition_dialogue_manager.natex_nlu import NatexNLU
+from emora_stdm.state_transition_dialogue_manager.natex_common import *
 from typing import Union, Set, List, Dict, Callable, Tuple, NoReturn, Any
 import nltk
 from spacy.pipeline import EntityRecognizer
@@ -437,4 +438,146 @@ class TokLimit(Macro):
         if ngrams.text().split() <= int(args[0]):
             return True
         else:
-            return False
\ No newline at end of file
+            return False
+
+class Agree(Macro):
+    """
+
+    """
+    def __init__(self):
+        self.natex = NatexNLU(agree)
+        self.natex.compile()
+        self.natex._regex = self.natex.regex().replace("_END_","").strip()
+
+    def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
+        return self.natex.regex()
+
+class Disagree(Macro):
+    """
+
+    """
+    def __init__(self):
+        self.natex = NatexNLU(disagree)
+        self.natex.compile()
+        self.natex._regex = self.natex.regex().replace("_END_","").strip()
+
+    def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
+        return self.natex.regex()
+
+class Question(Macro):
+    """
+
+    """
+    def __init__(self):
+        self.natex = NatexNLU(question)
+        self.natex.compile()
+        self.natex._regex = self.natex.regex().replace("_END_","").strip()
+
+    def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
+        return self.natex.regex()
+
+class Negation(Macro):
+    """
+
+    """
+    def __init__(self):
+        self.natex = NatexNLU(negation)
+        self.natex.compile()
+        self.natex._regex = self.natex.regex().replace("_END_","").strip()
+
+    def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
+        return self.natex.regex()
+
+class DontKnow(Macro):
+    """
+
+    """
+
+    def __init__(self):
+        dont_know = '[{' \
+                    'dont know,do not know,unsure,[not,{sure,certain}],hard to say,no idea,uncertain,[!no {opinion,opinions,idea,ideas,thought,thoughts,knowledge}],' \
+                    '[{dont,do not}, have, {opinion,opinions,idea,ideas,thought,thoughts,knowledge}],' \
+                    '[!{cant,cannot,dont} {think,remember,recall}]' \
+                    '}]'
+        self.natex = NatexNLU(dont_know)
+        self.natex.compile()
+        self.natex._regex = self.natex.regex().replace("_END_", "").strip()
+
+
+    def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
+        return self.natex.regex()
+
+
+class Maybe(Macro):
+    """
+
+    """
+
+    def __init__(self):
+        maybe = '[{maybe,possibly,sort of,kind of,kinda,a little,at times,sometimes,could be,potentially,its possible}]'
+        self.natex = NatexNLU(maybe)
+        self.natex.compile()
+        self.natex._regex = self.natex.regex().replace("_END_", "").strip()
+
+    def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
+        return self.natex.regex()
+
+class Transition(Macro):
+    """
+
+    """
+    def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
+        if len(args) > 1:
+            vars['__transition_score__'] = float(args[1])
+        else:
+            vars['__transition_score__'] = 0.5
+        vars['__transition__'] = args[0]
+        vars['__converged__'] = 'True'
+
+class Unexpected(Macro):
+
+    def __init__(self):
+        self.question_natex = NatexNLU(question)
+
+    def run(self, ngrams: Ngrams, vars: Dict[str, Any], args: List[Any]):
+        if self.question_natex.match(ngrams.text()):
+            if '_explained_stupidity_' in vars and vars['_explained_stupidity_'] == 'True':
+                vars['__response_prefix__'] = 'Placeholder I don\'t understand'
+            else:
+                vars['_explained_stupidity_'] = 'True'
+                vars['__response_prefix__'] = 'Placeholder I don\'t know'
+        else:
+            vars['__response_prefix__'] = 'Yeah.'
+        return True
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
