@@ -4,7 +4,7 @@ import json, os
 import random
 
 # Xiangjue Dong
-# 2/18/2020
+# 3/21/2020
 
 # states are typically represented as an enum
 
@@ -50,6 +50,12 @@ class State(Enum):
     CITY_CULTURE_Y = 51
     CITY_CULTURE_N = 52
 
+    CITY_REASON = 60
+    CITY_REASON_POSITIVE = 61
+    CITY_REASON_NEGATIVE = 62
+    CITY_REASON_UNKNOWN = 63
+    CITY_REASON_ANS = 64
+
 
 class TRAVEL_CATCH(Macro):
     """Catch user utterance
@@ -85,64 +91,49 @@ class TRAVEL_RANDOM(Macro):
     """
     def __init__(self, path):
         self.path = path
-        self.db_keys = []
-        self.db_keys_1 = []
-        self.db_keys_2 = []
-        self.db_keys_3 = []
         with open(self.path, 'r') as f:
             self.db = json.load(f)
 
     def run(self, ngrams, vars, args):
         # Random generate the first key
+        name = 'db_keys' + self.path
+        name_1 = 'db_keys_1' + self.path
+        name_2 = 'db_keys_2' + self.path
+        name_3 = 'db_keys_3' + self.path
+
         if len(args) == 0:
-            if len(self.db_keys) == 0:
-                self.db_keys = list(self.db.keys())
-                key = random.choice(self.db_keys)
-                self.db_keys = self.db_keys.remove(key)
-            else:
-                key = random.choice(self.db_keys)
-                self.db_keys = self.db_keys.remove(key)
+            if vars.get(name) is None or len(vars[name]) == 0:
+                vars[name] = list(self.db.keys())
+            key = random.choice(vars[name])
+            vars[name] = vars[name].remove(key)
             return key
 
         # Random generate unduplicated the first key
         elif len(args) == 1:
-            if len(self.db_keys_1) == 0:
-                self.db_keys_1 = list(self.db[vars[args[0]]].keys())
-                if vars[args[0]] in self.db_keys_1:
-                    self.db_keys_1.remove(vars[args[0]])
-                key_1 = random.choice(self.db_keys_1)
-                self.db_keys_1 = self.db_keys_1.remove(key_1)
-            else:
-                if vars[args[0]] in self.db_keys_1:
-                    self.db_keys_1.remove(vars[args[0]])
-                key_1 = random.choice(self.db_keys_1)
-                self.db_keys_1 = self.db_keys_1.remove(key_1)
+            if vars.get(name_1) is None or len(vars[name_1]) <= 1:
+                vars[name_1] = list(self.db[vars[args[0]]].keys())
+            if vars[args[0]] in vars[name_1]:
+                vars[name_1].remove(vars[args[0]])
+            key_1 = random.choice(vars[name_1])
+            vars[name_1] = vars[name_1].remove(key_1)
             return key_1
 
         # Random generate the third key
         elif len(args) == 2:
-            if len(self.db_keys_2) == 0:
-                self.db_keys_2 = list(self.db[vars[args[0]]][args[1]].keys())
-                key_2 = random.choice(self.db_keys_2)
-                self.db_keys_2 = self.db_keys_2.remove(key_2)
-            else:
-                key_2 = random.choice(self.db_keys_2)
-                self.db_keys_2 = self.db_keys_2.remove(key_2)
+            if vars.get(name_2) is None or len(vars[name_2]) <= 1:
+                vars[name_2] = list(self.db[vars[args[0]]][args[1]].keys())
+            key_2 = random.choice(vars[name_2])
+            vars[name_2] = vars[name_2].remove(key_2)
             return key_2
 
         # Random generate unduplicated the third key
         elif len(args) == 3:
-            if len(self.db_keys_3) == 0:
-                self.db_keys_3 = list(self.db[vars[args[0]]][vars[args[1]]].keys())
-                if vars[args[-1]] in self.db_keys_3:
-                    self.db_keys_3.remove(vars[args[0]])
-                key_3 = random.choice(self.db_keys_3)
-                self.db_keys_3 = self.db_keys_3.remove(key_3)
-            else:
-                if vars[args[-1]] in self.db_keys_3:
-                    self.db_keys_3.remove(vars[args[0]])
-                key_3 = random.choice(self.db_keys_3)
-                self.db_keys_3 = self.db_keys_3.remove(key_3)
+            if vars.get(name_3) is None or len(vars[name_3]) <= 1:
+                vars[name_3] = list(self.db[vars[args[0]]][vars[args[1]]].keys())
+            if vars[args[-1]] in vars[name_3]:
+                vars[name_3].remove(vars[args[0]])
+            key_3 = random.choice(vars[name_3])
+            vars[name_3] = vars[name_3].remove(key_3)
             return key_3
 
 
@@ -170,6 +161,7 @@ class TRAVEL_DETAIL(Macro):
         elif len(args) == 3:
             return self.db[vars[args[0]]][args[1]][vars[args[2]]]
 
+
 class CATCH_LIST(Macro):
     """Catch user utterance with list.
 
@@ -185,21 +177,24 @@ class CATCH_LIST(Macro):
         """Performs operation"""
         return ngrams & self.list
 
+
 # Database
 travel_db = os.path.join('modules','travel_database.json')
 
 # Variables
 TRANSITION_OUT = ["movies", "music", "sports"]
 NULL = "NULL TRANSITION"
+# The cities not in the database
 CITY_LIST = {"tokyo","jakarta","chongqing","manila","delhi","seoul","mumbai","shanghai","sao paulo","beijing",
-             "lagos","mexico city","guangzhou","dhaka","osaka","cairo","karachi","moscow","bangkok","chengdu",
-             "kolkata","buenos aires","istanbul","tehran","london","shenzhen","tianjin","kinshasa","rio de janeiro", "paris",
+             "lagos","mexico city","guangzhou","dhaka","osaka","cairo","karachi","moscow","chengdu",
+             "kolkata","buenos aires","istanbul","tehran","shenzhen","tianjin","kinshasa","rio de janeiro",
              "baoding", "lahore", "lima", "bangalore", "ho chi minh", "harbin", "wuhan", "shijiazhuang", "bogota", "suzhou",
              "linyi", "chennai", "nagoya", "nanyang", "zhengzhou", "hyderabad", "surabaya", "hangzhou", "johannesburg",
-             "chicago", "quanzhou", "taipei", "dongguan", "bandung", "hanoi", "shenyang", "baghdad", "onitsha", "kuala lumpur",
-             "ahmedabad", "luanda", "washington dc", "dallas", "hong kong", "pune", "nanjing", "boston", "santiago",
+             "chicago", "quanzhou", "taipei", "dongguan", "bandung", "hanoi", "shenyang", "baghdad", "onitsha",
+             "ahmedabad", "luanda", "washington dc", "dallas", "pune", "nanjing", "boston", "santiago",
              "riyadh", "dusseldorf", "madrid", "toronto", "surat"}
-brief_intro = "brief_intro"
+YES = {"yes", "yea", "yup", "yep", "i do", "yeah", "a little", "sure", "of course", "i have", "i am", "sometimes", "too", "as well", "also", "agree","good", "keep","why not", "ok", "okay", "fine", "continue", "go on"}
+NO = {"no", "nope", "dont", "nothing"}
 
 # Functions
 macros = {
@@ -211,6 +206,8 @@ macros = {
     'RANDOM_CULTURE': TRAVEL_RANDOM(travel_db),
     'DETAIL': TRAVEL_DETAIL(travel_db),
     'CATCH_CITY_LIST': CATCH_LIST(CITY_LIST),
+    'CATCH_YES':CATCH_LIST(YES),
+    'CATCH_NO':CATCH_LIST(NO)
 }
 
 ###################### Initialization Part ####################################################################################################################
@@ -223,7 +220,7 @@ df = DialogueFlow(State.START, initial_speaker=DialogueFlow.Speaker.USER, macros
 # For dialogue manager initialization
 # test
 # df.add_user_transition(State.START, State.START_TRAVEL, 'test')
-df.add_system_transition(State.START_TRAVEL, State.ASK_TRAVEL,'"Last month, one of my friends went to"$city={seattle, houston, atlanta}".i was curious about this. Which city is your dream city for travelling?"')
+df.add_system_transition(State.START_TRAVEL, State.ASK_TRAVEL,'"Last month, one of my friends went to"$city={seattle, houston, atlanta}". Which city is your dream city for travelling?"')
 
 # User Turn
 df.add_user_transition(State.START, State.TRAVEL_Y, '[{travel, traveling, city, cities}]')
@@ -235,7 +232,7 @@ df.set_error_successor(State.START, State.START)
 df.add_system_transition(State.START, State.START, NULL)
 
 # System Turn
-df.add_system_transition(State.TRAVEL_Y, State.ASK_TRAVEL, '"Awesome! Last month, one of my friends went to"$city={seattle, houston, atlanta}"and she liked there very much. I know several wonderful cities to travel to in the United States, which city do you want to go in the United States?"')
+df.add_system_transition(State.TRAVEL_Y, State.ASK_TRAVEL, '"Awesome! Last month, one of my friends went to"$city={seattle, houston, atlanta}"and she liked there very much. I know several wonderful cities to travel to in the United States. Which city do you want to go in the United States?"')
 
 # User Turn
 df.add_user_transition(State.ASK_TRAVEL, State.ASK_TRAVEL_CITY, '[$city=#CATCH()]')
@@ -251,7 +248,7 @@ df.add_system_transition(State.ASK_TRAVEL_CITY_UNKNOWN_INLIST, State.CITY_TOURIS
 
 ############################# Tourist Attraction Part ############################################################################################################
 # User Turn
-df.add_user_transition(State.CITY_TOURISM, State.CITY_TOURISM_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, fine, okay, tourist attraction, tourist, tourism, attraction, attraction, tour, good, keep, why not}]')
+df.add_user_transition(State.CITY_TOURISM, State.CITY_TOURISM_Y, '[{#CATCH_YES(),tourist attraction, tourist, tourism, attraction, tour}]')
 df.add_user_transition(State.CITY_TOURISM, State.ASK_TRAVEL_CITY, '[$city=#CATCH()]')
 df.add_user_transition(State.CITY_TOURISM, State.ASK_TRAVEL_CITY_UNKNOWN_INLIST, '[$city=#CATCH_CITY_LIST()]')
 # df.add_user_transition(State.CITY_TOURISM, State.CITY_TOURISM_CERTAIN_N, '[[{no, nope, dont}] #NOT(know, idea)]')
@@ -264,8 +261,8 @@ df.add_system_transition(State.CITY_TOURISM_Y, State.CITY_TOURISM_CERTAIN, '"Goo
 # df.add_system_transition(State.CITY_TOURISM_UNKNOWN, State.CITY_TOURISM, '"Interesting. I am not quite familiar with this aspect of"$city"What about talking about something that I know, like tourist attraction there?"')
 
 # User Turn
-df.add_user_transition(State.CITY_TOURISM_CERTAIN, State.CITY_TOURISM_CERTAIN_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, fine, okay, tourist attraction, tourist, tourism, attraction, attraction, tour}]')
-df.add_user_transition(State.CITY_TOURISM_CERTAIN, State.CITY_TOURISM_CERTAIN_N, '[{no, nope, dont}]')
+df.add_user_transition(State.CITY_TOURISM_CERTAIN, State.CITY_TOURISM_CERTAIN_Y, '[#CATCH_YES()]')
+df.add_user_transition(State.CITY_TOURISM_CERTAIN, State.CITY_TOURISM_CERTAIN_N, '[#CATCH_NO()]')
 # df.add_user_transition(State.CITY_TOURISM_CERTAIN, State.ASK_TRAVEL_CITY, '[$city=#CATCH()]')
 df.set_error_successor(State.CITY_TOURISM_CERTAIN, State.CITY_TOURISM_CERTAIN_N)
 
@@ -275,7 +272,7 @@ df.add_system_transition(State.CITY_TOURISM_CERTAIN_N, State.CITY_FOOD, '{Alrigh
 
 ############################# Famous Food Part #####################################################################################################################
 # User Turn
-df.add_user_transition(State.CITY_FOOD, State.CITY_FOOD_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, fine, okay, tourist attraction, tourist, tourism, attraction, attraction, tour, good, keep, why not}]')
+df.add_user_transition(State.CITY_FOOD, State.CITY_FOOD_Y, '[#CATCH_YES()]')
 # df.add_user_transition(State.CITY_FOOD, State.CITY_FOOD_CERTAIN_N, '[[{no, nope, dont}] #NOT(know, idea)]')
 df.add_user_transition(State.CITY_FOOD, State.ASK_TRAVEL_CITY, '[$city=#CATCH()]')
 df.add_user_transition(State.CITY_FOOD, State.ASK_TRAVEL_CITY_UNKNOWN_INLIST, '[$city=#CATCH_CITY_LIST()]')
@@ -287,8 +284,8 @@ df.add_system_transition(State.CITY_FOOD_Y, State.CITY_FOOD_CERTAIN, '"I like fo
 df.add_system_transition(State.CITY_FOOD_UNKNOWN, State.CITY_FOOD, '"Interesting. I am not quite familiar with this aspect of"$city"What about talking about something that I know, like famous food there?"')
 
 # User Turn
-df.add_user_transition(State.CITY_FOOD_CERTAIN, State.CITY_FOOD_CERTAIN_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, fine, okay, famous food, food}]')
-df.add_user_transition(State.CITY_FOOD_CERTAIN, State.CITY_FOOD_CERTAIN_N, '[{no, nope, dont}]')
+df.add_user_transition(State.CITY_FOOD_CERTAIN, State.CITY_FOOD_CERTAIN_Y, '[#CATCH_YES()]')
+df.add_user_transition(State.CITY_FOOD_CERTAIN, State.CITY_FOOD_CERTAIN_N, '[#CATCH_NO()]')
 df.set_error_successor(State.CITY_FOOD_CERTAIN, State.CITY_FOOD_CERTAIN_N)
 
 # System Turn
@@ -297,7 +294,7 @@ df.add_system_transition(State.CITY_FOOD_CERTAIN_N, State.CITY_SPORTS, '{Alright
 
 ############################ Sports Part ###########################################################################################################################
 # User Turn
-df.add_user_transition(State.CITY_SPORTS, State.CITY_SPORTS_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, fine, okay, tourist attraction, tourist, tourism, attraction, attraction, tour, good, keep, why not}]')
+df.add_user_transition(State.CITY_SPORTS, State.CITY_SPORTS_Y, '[#CATCH_YES()]')
 # df.add_user_transition(State.CITY_SPORTS, State.CITY_SPORTS_N, '[[{no, nope, dont}] #NOT(know, idea)]')
 df.add_user_transition(State.CITY_SPORTS, State.ASK_TRAVEL_CITY, '[$city=#CATCH()]')
 df.add_user_transition(State.CITY_SPORTS, State.ASK_TRAVEL_CITY_UNKNOWN_INLIST, '[$city=#CATCH_CITY_LIST()]')
@@ -310,7 +307,7 @@ df.add_system_transition(State.CITY_SPORTS_N, State.CITY_EVENTS, '{Alright, Ok,
 
 ############################# Events Part ##########################################################################################################################
 # User Turn
-df.add_user_transition(State.CITY_EVENTS, State.CITY_EVENTS_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, fine, okay, tourist attraction, tourist, tourism, attraction, attraction, tour, good, keep, why not}]')
+df.add_user_transition(State.CITY_EVENTS, State.CITY_EVENTS_Y, '[#CATCH_YES()]')
 # df.add_user_transition(State.CITY_EVENTS, State.CITY_EVENTS_CERTAIN_N, '[[{no, nope, dont}] #NOT(know, idea)]')
 df.add_user_transition(State.CITY_EVENTS, State.ASK_TRAVEL_CITY, '[$city=#CATCH()]')
 df.add_user_transition(State.CITY_EVENTS, State.ASK_TRAVEL_CITY_UNKNOWN_INLIST, '[$city=#CATCH_CITY_LIST()]')
@@ -333,16 +330,30 @@ df.add_system_transition(State.CITY_EVENTS_CERTAIN_N, State.CITY_CULTURE, '{Alri
 
 ############################# Culture Part #########################################################################################################################
 # User Turn
-df.add_user_transition(State.CITY_CULTURE, State.CITY_CULTURE_Y, '[{yes, yea, yup, yep, i do, yeah, a little, sure, of course, i have, i am, sometimes, too, as well, also, agree, ok, fine, okay, tourist attraction, tourist, tourism, attraction, attraction, tour, good, keep, why not}]')
+df.add_user_transition(State.CITY_CULTURE, State.CITY_CULTURE_Y, '[#CATCH_YES()]')
 df.set_error_successor(State.CITY_CULTURE, State.CITY_CULTURE_N)
 df.add_user_transition(State.CITY_CULTURE, State.ASK_TRAVEL_CITY, '[$city=#CATCH()]')
 df.add_user_transition(State.CITY_CULTURE, State.ASK_TRAVEL_CITY_UNKNOWN_INLIST, '[$city=#CATCH_CITY_LIST()]')
 
 # System Turn
-df.add_system_transition(State.CITY_CULTURE_Y, State.END, '{Nice,Good, Great, Cool,Wonderful,Awesome}"!" #DETAIL(city, culture, culture={#RANDOM_CULTURE(city,culture)})"I have to say"$city"is a wonderful city which is worth traveling because"#DETAIL(city, reason_for_travel)"!"')
-df.add_system_transition(State.CITY_CULTURE_N, State.END, '{Alright, Ok, Then}".But I have to say"$city"is a wonderful city which is worth traveling because"#DETAIL(city, reason_for_travel)"!"')
+df.add_system_transition(State.CITY_CULTURE_Y, State.CITY_REASON, '{Nice,Good, Great, Cool,Wonderful,Awesome}"!About the"$certain_culture={#RANDOM_CULTURE(city,culture)}","#DETAIL(city, culture, certain_culture)"So, why do you want to travel to"$city"?"')
+df.add_system_transition(State.CITY_CULTURE_N, State.CITY_REASON, '{Alright, Ok, Then}"So, why do you want to travel to"$city"?"')
+
+############################# Travel Reason ##########################################################################################################
+# User Turn
+df.add_user_transition(State.CITY_REASON, State.CITY_REASON_POSITIVE, '[{interesting, beautiful, weather}]')
+df.add_user_transition(State.CITY_REASON, State.CITY_REASON_NEGATIVE,'[#CATCH_NO()]')
+df.set_error_successor(State.CITY_REASON, State.CITY_REASON_UNKNOWN)
+
+# System Turn
+df.add_system_transition(State.CITY_REASON_POSITIVE, State.END, '{Nice,Good, Great, Cool,Wonderful,Awesome, I agree}"!I have to say"$city"is a wonderful city which is worth traveling to because it"#DETAIL(city, reason_for_travel)"!"')
+df.add_system_transition(State.CITY_REASON_NEGATIVE, State.END, '{Alright, Ok, Then}".But I think"$city"is a wonderful city which is worth traveling to because it"#DETAIL(city, reason_for_travel)"!"')
+df.add_system_transition(State.CITY_REASON_UNKNOWN, State.END, '{Interesting, I see}".In my opinion, I think"$city"is a wonderful city which is worth traveling to because it"#DETAIL(city, reason_for_travel)"!"')
 
 ####################### End Travel Component ##############################################################################################################################################
+# # END
+# df.set_error_successor(State.CITY_REASON_ANS, State.TRAVEL_END)
+# df.add_system_transition(State.TRAVEL_END, State.END, '"I\'m glad to talk with you. What other topics would you like to talk about?"')
 
 df.update_state_settings(State.END, system_multi_hop=True)
 # df.add_system_transition(State.END, State.START, '" "')
